clear all
clc

% This routine compute summarize statatics and normality test in Qian(2011) 
% "Bayesian Portfolio Selection in Markov Switching Gaussian Mixture Model"

myseed = 20111218;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
end

[price,header] = xlsread('StockPrice.xls');
lnP = log(price);
R = diff(lnP)*100;

% descriptive statistics
[nobs,dim] = size(R);
R_mean = mean(R);
R_cov = cov(R);
R_skew = skewness(R);
R_kurtosis =  kurtosis(R);

fprintf('mean \t')
fprintf('%8.2f', R_mean)
fprintf('\n')
fprintf('skew \t')
fprintf('%8.2f', R_skew)
fprintf('\n')
fprintf('kurt \t')
fprintf('%8.2f', R_kurtosis)
fprintf('\n')
fprintf('\n')

disp('covariance matrix: ')
for m = 1:dim
    fprintf('%8.2f', R_cov(m,:))
    fprintf('\n')
end
fprintf('\n')


% Nomality test: Lilliefors (empirical KS); Jarque-Bera
stat_LI = zeros(1,dim);
p_LI = zeros(1,dim);
stat_JB = zeros(1,dim);
p_JB = zeros(1,dim);
for m = 1:dim
    [accept_LI,p_LI(m),stat_LI(m)]  =  lillietest(R(:,m));
    [accept_JB,p_JB(m),stat_JB(m)]  =  jbtest(R(:,m));
end

disp('Lilliefors (empirical KS) test: ')
fprintf('%9.3f', stat_LI)
fprintf('\n')
fprintf('%9.3f', p_LI)
fprintf('\n')
fprintf('\n')

disp('Jarque-Bera test: ')
fprintf('%10.1f', stat_JB)
fprintf('\n')
fprintf('%10.3f', p_JB)
fprintf('\n')
fprintf('\n')

% Estimate the model under the normality assumption
ndraws = 20000;
burn_in = ndraws * 0.5;
[Mu_draws,Sigma_draws,Markov_draws,Initial_prob_draws,MixProb_draws,Regime_draws,R_predict] = HMM_GM1(R,1,1,ndraws,burn_in);
ndraws = ndraws - burn_in;
Mu_draws = reshape(Mu_draws,dim,ndraws);
Sigma_draws = reshape(Sigma_draws,dim,dim,ndraws);
post_mean = mean(Mu_draws,2);
post_var = diag(mean(Sigma_draws,3));
pred_mean = mean(R_predict,2);
pred_var = var(R_predict,0,2);

% Bayesian residual test
stat_KS = zeros(1,ndraws);
figure (1)
for m = 1:dim    
       
    for n = 1:ndraws
        resid = (R(:,m)-Mu_draws(m,n))/sqrt(Sigma_draws(m,m,n));
        [accept_KS,p_KS,stat_KS(n)]  =  kstest(resid);
    end    
    subplot(3,2,m),hist(stat_KS,20); 
    
    % KS approximated 99% critical value 1.63/sqrt(n)
    critical_val = 1.63/sqrt(nobs);
    percentile = mean(stat_KS>critical_val)*100;
    disp(['R(',num2str(m),'): ',num2str(percentile),'% simulators exceed 99% KS critical value'])
    
end


% Gererate a report
header = header(1,2:end);
xlswrite('Descriptive Statistics',header,'Sheet1','B1')
xlswrite('Descriptive Statistics',{'Mean';'Skewness';'Kurtosis';'';'Lilliefors';'p-val';'';'Jarque-Bera';'p-val'},'Sheet1','A2')
xlswrite('Descriptive Statistics',[R_mean;R_skew;R_kurtosis;nan(1,dim);stat_LI;p_LI;nan(1,dim);stat_JB;p_JB],'Sheet1','B2')

xlswrite('Descriptive Statistics',header,'Sheet2','B1')
xlswrite('Descriptive Statistics',header','Sheet2','A2')
xlswrite('Descriptive Statistics',R_cov,'Sheet2','B2')

xlswrite('Descriptive Statistics',header,'Sheet3','B1')
xlswrite('Descriptive Statistics',{'Post. mean';'Post. var';'';'Pred. mean';'Pred. var'},'Sheet3','A2')
xlswrite('Descriptive Statistics',[post_mean';post_var';nan(1,dim);pred_mean';pred_var'],'Sheet3','B2')