function [Mu_draws,Sigma_draws,Markov_draws,Initial_prob_draws,MixProb_draws,Regime_draws,Y_predict] = HMM_GM_EQ(Y,nregime,nstate,ndraws,burn_in,permute_flag)

% Purpose: 
% Estimate Markov Switching Gaussian Mixture (MSGM) model using Gibbs sampler
% In this version, covariance matrix is invariant across regimes and states
% -----------------------------------
% Model:
% Latent regimes 1,..., R follow Markov chain law of motion (Q).
% Conditional on the regime, what is observed is 
% a k dimensional multivarite Gaussian mixture with S states
% The mean and covariance matrix depends on the latent regime
% Prior distribution:
% Mu(s,r) ~ N(mu(s,r),V(s,r))
% Sigma ~ IW(v,Omega)
% Q(r) ~ Dirichlet(a1(r),...,aR(r))
% pi(r) ~ Dirichlet(c1,...,cR)
% where the regimes r = 1,...,R, the states s = 1,...,S
% -----------------------------------
% Algorithm:
% 1.Sample posterior latent regimes by method of composition
%   with recursive computation of Baum-Welch backward variables
%   Other parameters have conjugate posterior conditionals.
% 2. Geweke (2007) permutation augmented posterior simulator
% -----------------------------------
% Usage:
% Y = Observed data (T*k matrix)
% nregime = Number of latent Markov switching regimes (R)
% nstate = Number of latent states of Gaussian mixture (S)
% ndraws = Number of draws in MCMC (J+B)
% burn_in = Number of burn-in draws in MCMC (B)
% permute_flag: permute current draws in desired order
%   0 = no permutation
%   1 = permute by average coefficients of the MVN mean vector
%   2 = permute by average variance of the MVN
% -----------------------------------
% Returns:
% Mu_draws = posterior draws of mean vector of the normal distribution under each regime (k*S*R*J array)
% Sigma_draws = posterior draws of covariance matrix of the normal distribution under each regime (k*k*S*R*J array)
% Markov_prob_draws = posterior draws of transition matrix of (vectorized) Markov chain (R*R*J array)
% Initial_prob_draws = posterior draws of probability of regimes at Date 1 (R*J matrix)
% MixProb_draws = posterior draws of the mixing probability in the Gaussian mixture (S*R*J arragy)
% Regime_draws = posterior draws of the last regime (1 * J vector)
% Y_predict = Predict Y from its posterior predictive distribution (k*J matrix)
% where the remaining number of draws J = ndraws - burn_in
% -----------------------------------
% Notes:
% The process starts from Date 1, a latent regime is drawn from the initial distribution,
% and then a signal is generated conditional on that regime.
%
% Written by Hang Qian, Iowa Regime University
% Contact me:  matlabist@gmail.com

if nargin < 2;  nregime = 2;disp('For illustration, assume two latent Markov switching regimes.'); end
if nargin < 3;  nstate = 2;disp('For illustration, assume Gaussian mixture contain two states.'); end
if nargin < 4;  ndraws = 10000; disp(['Number of draws equals ',num2str(ndraws)]);  end
if nargin < 5;  burn_in = 0.5 * ndraws;disp(['Number of burn_in draws equals ',num2str(burn_in)]);  end
if nargin < 6;  permute_flag = 0;    end


try
    JustTest = gamrnd(3,1);
catch
    disp(' ')
    disp('Oooopse, Matlab statistics toolbox is not installed.')
    disp('You may download a compatibility package on my website.')
    disp('http://www.public.iaregime.edu/~hqi/toolkit')
    error('Program exits.')
end

% Analyze model structure
[nobs, nvar] = size(Y);
if nobs < nvar
    Y = Y';
    [nobs, nvar] = size(Y);
end


% --------------------------------------------------------
% Prior distribution settings
% Mu(s,r) ~ N(mu(s,r),V(s,r))
% Sigma ~ IW(v,Omega)
% Q(r) ~ Dirichlet(a1(r),...,aR(r))
% pi ~ Dirichlet(c1,...,cR)
% MixProb(r) ~ Dirichlet(f1(r),...,fS(r))
% You may change the hyperparameters here if needed
prior_mu = zeros(nvar,nstate,nregime);
prior_V = zeros(nvar,nvar,nstate,nregime);
for r = 1:nregime
    for s = 1:nstate
        prior_V(:,:,s,r) = 100 * eye(nvar);        
    end
end
prior_Omega = eye(nvar);
prior_df = nvar + 5;
prior_Markov = ones(nregime,nregime);
prior_Initial = ones(nregime,1);
prior_MixProb = ones(nstate,nregime);
% --------------------------------------------------------


% Preallocate memory
Mu_draws = zeros(nvar,nstate,nregime,ndraws - burn_in);
Sigma_draws = zeros(nvar,nvar,ndraws - burn_in);
Markov_draws = zeros(nregime,nregime,ndraws - burn_in);
Initial_prob_draws = zeros(nregime,ndraws - burn_in);
MixProb_draws = zeros(nstate,nregime,ndraws - burn_in);
Regime_draws = zeros(1,ndraws - burn_in);
Y_predict = zeros(nvar,ndraws - burn_in);

freq = zeros(nregime,nregime);
normpdf_rec = zeros(nregime,nobs);
Backward = zeros(nregime,nobs);
Backward(:,nobs) = 1;
npermute_state = 0;
npermute_regime = 0;

% Initial draws
Mu_use = zeros(nvar,nstate,nregime);
for r = 1:nregime
    for s = 1:nstate
        Mu_use(:,s,r) = prior_mu(:,s,r) + chol(prior_V(:,:,s,r))' * randn(nvar,1);        
    end
end
Sigma_inv_use = WISHART_RND(prior_Omega,prior_df);
Sigma_use = inv(Sigma_inv_use);
Markov_prob = ones(nregime,nregime) ./ nregime;
Initial_prob = Markov_prob(1,:);
MixProb = ones(nstate,nregime) ./ nstate;
Regime_use = ceil(rand(nobs,1) * nregime);
State_use = ceil(rand(nobs,1) * nstate);


% Gibbs sampler
H_status_bar = waitbar(0,'MCMC in progress 0%');
percentage = floor(ndraws / 100);
TotalTime = 0;
tic
for j = 1:ndraws         
    
    % You are welcome to copy this status bar into your own codes,
    % but please also include this line: Designed by Hang Qian
    if mod(j,percentage) == 0
        Elapse = toc;
        TotalTime = TotalTime + Elapse;
        RemainTime = TotalTime * (ndraws - j) / j;
        if RemainTime > 3600
            RemainTime = round(RemainTime/3600);
            waitbar(j / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(j / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' h']);
        elseif RemainTime > 60
            RemainTime = round(RemainTime/60);
            waitbar(j / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(j / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' m']);
        else
            RemainTime = round(RemainTime);
            waitbar(j / ndraws,H_status_bar,['MCMC in Progress ',num2str(floor(j / ndraws * 100)),'%   Time Remaining ',num2str(RemainTime),' s']);
        end
        tic
    end
    
    % Sample Mu, Sigma, Markov transition matrix, Mixing Prob regime by regime
    Tsr = zeros(nstate,nregime);
    resid = zeros(nobs,nvar);
    for r = 1:nregime
        for s = 1:nstate
            % Sample posterior conditional draws from Mu|. ~ N(Dd,D)
            mask = (Regime_use == r) & (State_use == s);
            Tsr(s,r) = sum(mask);
            Ysr = Y(mask,:);
            D = inv(Tsr(s,r) .* Sigma_inv_use + inv(prior_V(:,:,s,r)) );
            d = Sigma_inv_use * sum(Ysr,1)' + prior_V(:,:,s,r) \ prior_mu(:,s,r);
            Mu_use(:,s,r) = D * d + chol(D)' * randn(nvar,1);
            
            % Sample posterior conditional draws from sigma2|. ~ IG(a_up,b_up)
            resid(mask,:) = Ysr - repmat(Mu_use(:,s,r)',Tsr(s,r),1);
            % RSS = resid' * resid;
            % post_Omega = inv(inv(prior_Omega(:,:,s,r)) + RSS);
            % post_df = prior_df(s,r) + Tsr(s,r);
            % Sigma_inv_use(:,:,s,r) = WISHART_RND(post_Omega,post_df);
            % Sigma_use(:,:,s,r) = inv(Sigma_inv_use(:,:,s,r));
        end
                
        % Sample mixing probability from conjugate Dirichlet distribution
        MixProb(:,r) = DIRICHLET_RND(prior_MixProb(:,r) + Tsr(:,r));
        
        % Sample posterior conditional draws for Markov transition matrix
        % It follows conjugate Dirichlet
        mask = (Regime_use == r);
        mask_next = [false;mask(1:nobs-1)];
        Regime_next = Regime_use(mask_next);
        for k = 1:nregime
            freq(r,k) = sum(Regime_next == k);
        end
        post_Markov = prior_Markov(r,:) + freq(r,:);
        Markov_prob(r,:) = DIRICHLET_RND(post_Markov,1);
    end
    Markov_prob_tr = Markov_prob'; 
    
    % Sample posterior conditional draws from sigma2|. ~ IG(a_up,b_up)    
    RSS = resid' * resid;
    post_Omega = inv(inv(prior_Omega) + RSS);
    post_df = prior_df + nobs;
    Sigma_inv_use = WISHART_RND(post_Omega,post_df);
    Sigma_use = inv(Sigma_inv_use);
        
    % Sample posterior conditional draws for initial distribution
    % It follows conjugate Dirichlet
    credit = zeros(nregime,1);
    credit(Regime_use(1)) = 1;
    post_Initial = prior_Initial + credit;
    Initial_prob = DIRICHLET_RND(post_Initial,1); 
    
    % Permute parameters for desired order
    switch permute_flag
        case 1
            % Permute states within each regime
            for r = 1:nregime
                [val,ind] = sort(mean(Mu_use(:,:,r),1));
                if any(ind ~= (1:nstate))
                    Mu_use(:,:,r) = Mu_use(:,ind,r);
                    MixProb(:,r) = MixProb(ind,r);
                    npermute_state = npermute_state + 1;
                end
            end
            
            % Permute regime itself
            Mu_use_avg = mean(Mu_use(:,end,:),1);
            [val,ind] = sort(Mu_use_avg(:));
            if any(ind ~= (1:nregime)')
                Mu_use = Mu_use(:,:,ind);                
                MixProb = MixProb(:,ind);
                Markov_prob = Markov_prob(ind,ind);
                Initial_prob = Initial_prob(ind);
                Markov_prob_tr = Markov_prob';
                npermute_regime = npermute_regime + 1;
            end        
    end
       
    
    %------------
    % Core codes
    %------------
    % Sample latent regimes by method of composition
    % Baum-Welch posterior regimes probability algorithm is used     

    % Compute pdf of observed signal in each regime
    density_big = zeros(nobs,nstate,nregime);
    for r = 1:nregime
        density = zeros(nobs,nstate);
        for s = 1:nstate
            density(:,s) = mvnpdf(Y,repmat(Mu_use(:,s,r)',nobs,1),Sigma_use);
        end
        MixDensity = density * MixProb(:,r);
        normpdf_rec(r,:) = MixDensity';
        density_big(:,:,r) = density;
    end

    
    % Compute recursively Baum-Welch backward variable
    % For numerical stabibility, normalize backward variable at each date
    % No need to keep track of the normalization constant
    % since we need to renormalize conditional probability before sampling
    
    % Interestingly, Gibbs sampler does not require forward variable
    % since it can be merged into the proportionality constant
    %     Forward(:,1) = Initial_prob .* normpdf_rec(:,1);    
    %     for t = 2:nobs
    %         % Forward_use = (Forward(:,t-1)' * Markov_prob)' .* normpdf_rec(:,t);
    %         Forward_use = Markov_prob_tr * Forward(:,t-1) .* normpdf_rec(:,t);       
    %         Forward(:,t) = Forward_use ./ sum(Forward_use);
    %     end    
    
    Backward(:,nobs-1) = Markov_prob * normpdf_rec(:,nobs);    
    for t = nobs-2:-1:1
        Backward_use = Markov_prob * (normpdf_rec(:,t+1) .* Backward(:,t+1));        
        Backward(:,t) = Backward_use ./ sum(Backward_use);        
    end
    
    % First of all, sample P(R(1)=r|theta) = F(1,r) * B(1,r)
    prob = Initial_prob .* normpdf_rec(:,1) .* Backward(:,1);
    cum_prob = cumsum(prob);
    cum_prob = cum_prob ./ cum_prob(end);
    Regime_use(1) = find(cum_prob > rand, 1);
    
    % Then, use method of composition to recursively sample 
    % P(R(t)=r|R(t-1)=s,theta) = F(t-1,s) * Markov(s,r) * phi(Y(t)|r) * B(t,r)
    % However, F(t-1,s) has nothing to do with current regime r,
    % so it can be merged into the porportionality constant
    for t = 2:nobs
        r = Regime_use(t-1);
        % prob = Forward(r,t-1) .* Markov_prob(r,:)' .* normpdf_rec(:,t) .* Backward(:,t);
        prob = Markov_prob_tr(:,r) .* normpdf_rec(:,t) .* Backward(:,t);
        cum_prob = cumsum(prob);
        cum_prob = cum_prob ./ cum_prob(end);
        Regime_use(t) = find(cum_prob > rand, 1);
    end
    
           
    % Sample latent states of the Gaussian mixture
    density_state = zeros(nobs,nstate);
    for t = 1:nobs
        density_state(t,:) = density_big(t,:,Regime_use(t)) .* MixProb(:,Regime_use(t))';
    end     
    W_sum = sum(density_state,2);
    W = density_state ./ W_sum(:,ones(nstate,1));
    cum_W = cumsum(W,2);    
    U = rand(nobs,1);
    U_matrix = U(:,ones(nstate,1));
    State_use = sum(cum_W<U_matrix,2) + 1;
       
    
    if j > burn_in
        % Record the draws
        Mu_draws(:,:,:, j - burn_in) = Mu_use;
        Sigma_draws(:,:, j - burn_in) = Sigma_use;        
        Markov_draws(:,:, j - burn_in) = Markov_prob;
        Initial_prob_draws(:, j - burn_in) = Initial_prob;
        MixProb_draws(:,:, j - burn_in) = MixProb;
        Regime_draws(:, j - burn_in) = Regime_use(end);
        
        % Make a prediction (Sample from posterior predictive distribution)
        Regime_predict = DISCRETE_RND(1:nregime,Markov_prob(Regime_use(end),:),1);
        State_predict = DISCRETE_RND(1:nstate,MixProb(:,Regime_predict)',1);
        chol_P = chol(Sigma_use);
        Y_predict(:,j - burn_in) = Mu_use(:,State_predict,Regime_predict) + chol_P' * randn(nvar,1);        
    end
    
end

% Summarize posterior distribution
Beta_mean = mean(Mu_draws,4);
Beta_std = std(Mu_draws,0,4);
Sigma_mean = mean(Sigma_draws,3);
Sigma_std = std(Sigma_draws,0,3);
Markov_mean = mean(Markov_draws,3);
Markov_std = std(Markov_draws,0,3);
Initial_prob_mean = mean(Initial_prob_draws,2);
Initial_prob_std = std(Initial_prob_draws,0,2);
MixProb_mean = mean(MixProb_draws,3);
MixProb_std = std(MixProb_draws,0,3);
Y_predict_mean = mean(Y_predict,2);
Y_predict_std = std(Y_predict,0,2);
eval([char([81 72 49 61]),'[87 114 105 116 116 101 110 32 98 121];'])
eval([char([81 72 50 61]),'[32 72 97 110 103 32 81 105 97 110];'])

result = cell(nvar + 1,3);
for r = 1:nregime
    disp(['-------------     Regime ',num2str(r),'    -------------'])    
    for s = 1:nstate
        disp(['    ***  State ',num2str(s),' ***'])
        disp({'Coef','Post. mean','Post. std'})
        for m = 1:nvar
            result(m,1) = {['Mu(',num2str(m),')']};
            result(m,2:3) = {Beta_mean(m,s,r),Beta_std(m,s,r)};
        end
        result(end,1) = {'Prob'};
        result(end,2:3) = {MixProb_mean(s,r),MixProb_std(s,r)};
        disp(result)
        disp(' ')     
        
    end    
end

disp(' ')
disp('------------------------')
disp('Covariance matrix (average)')
disp(Sigma_mean)
disp('with standard deviation')
disp(Sigma_std)
disp(' ')

disp(' ')
disp('------------------------')
disp('Markov transition matrix (mean)')
disp(Markov_mean)
disp('with standard deviation')
disp(Markov_std)
disp(' ')
disp('Initial probability vector (mean)')
disp(Initial_prob_mean')
disp('with standard deviation')
disp(Initial_prob_std')
disp(' ')
disp(['Number of regime permutations applied: ',num2str(npermute_regime)])
disp(' ')
disp(['Number of state permutations applied: ',num2str(npermute_state)])
disp(' ')

disp(' ')
disp('------------------------')
disp('Posterior predictive distribution')
disp(Y_predict_mean)
disp('with standard deviation')
disp(Y_predict_std)
disp(' ')
fwrite(1, char([QH1,QH2,10,13]))
close(H_status_bar);

