function [x,fval,exitflag,Gradient,Hessian] = MAXIMIZE(fun,x0,options)

% -----------------------------------
% Software Name:
% Climb High
% -----------------------------------
% Purpose: 
% Find the maximum of the user-supplied objective function
% -----------------------------------
% Algorithm: 
% A numerical routine for unconstrained non-linear optimization problems 
% on the basis of the BFGS algorithm
% -----------------------------------
% Usage:
% fun = user supplied objective function, 
%       which can be a function handle, anonymous function, or a character string
% x0 = starting value (if not specified, the program will try to generate one automatically)
% options = a structure containing optimization config. The fileds include 
%   MethodGradient: 
%       0 = user supplied analytic gradients (fun must return in the second output argument)
%       1 = numerical gradients with forward difference
%       2 = numerical gradients with central difference (default)
%       3 = numerical gradients with Richardson extrapolation of order 1
%       4 = numerical gradients with Richardson extrapolation of order 2
%       5 = numerical gradients with Richardson extrapolation of order 3
%       6 = numerical gradients with Richardson extrapolation on central difference
%   DiffDelta:  The small increment to compute numerical gradients ( default = 1e-6)
%   Display: 1 = display iteration results on screen (default), 0 = write into a plain text file
%   MaxIter: Maximum number of iterations (default = 200)
%   TolFun: Convergence criterion for both parameters and function values (default = 1e-6)
%   MaxAttempt: Maximum attempts to adjust line search window and and darts shoot (default = 20)
%   MaxDarts: In each attempt of darts shoot, the number of darts (default = 5)
%   NelderMead: If line search fails, whether switch to simplex search mode (default = 1)
%   Minutes: If x0 is not specified, the time allowing computer to search an initial value (default = 1, but not precise)
%   FinalCheck: Post convergence random check of the global maximum / minimum (default = 1)
%   Rounding: If final parameters are very close to integers, whether to round the results (default = 1)
% -----------------------------------
% Returns:
% x = Optimized parameters which maximize the objective function
% fval = maximum function value
% exitflag: 1 = full converge, 2 = fval & x converge, 3 = fval cannot improve, 4 = MaxIter reached, 0 = error occurs
% Gradient = Gradients evaluated at optimum
% Hessian = Hessian evaluated at optimum (Take time to compute. Do not add this output % argument if you do not need Hessian)
% -----------------------------------
% Examples:
% 1.Specify objective function using a function handle:
%   First create a function, say myfun():
%   y = myfun(x)
%   y = - (3 * x(1)^2 + 2 * x(1) * x(2) + x(2)^2)
%   Then you can use MAXIMIZE(myfun) to find optimum
% 2. Specify objective function using an anonymous function:
%   MAXIMIZE(@(x)  - (3 * x(1)^2 + 2 * x(1) * x(2)  + x(2)^2))
% 3. Specify objective function using character strings
%   MAXIMIZE('- (3 * x(1)^2 + 2 * x(1) * x(2)  + x(2)^2)')
%   MAXIMIZE('- (3 * x1^2 + 2 * x1 * x2  + x2^2)')
%   MAXIMIZE('- (3 * x^2 + 2 * x* y + y^2)')
% -----------------------------------
% Notes:
% Refer to Users Guide to Climb High for details on usage and algorithm
%
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com
% Copyright 2008-2011. All rights reserved.
% Release date: 07/23/2011


if nargin < 2
    x0 = [];
end


% Default values of optimization options
MethodGradient = 2;
DiffDelta = 1e-6;
Display = 1;
MaxIter = 200;
TolFun = 1e-6;
MaxAttempt = 20;
MaxDarts = 5;
NelderMead = 1;
Minutes = 1;
FinalCheck = 1;
Rounding = 1;

if nargin == 3
    field_flag = 0;
    if isfield(options,'MethodGradient') && isscalar(options.MethodGradient)
        MethodGradient = options.MethodGradient;
        disp(['User setting: MethodGradient = ',num2str(MethodGradient)])
        field_flag = 1;    
    end
    
    if isfield(options,'DiffDelta') && isscalar(options.DiffDelta)
        DiffDelta = options.DiffDelta;
        disp(['User setting: DiffDelta = ',num2str(DiffDelta)])
        field_flag = 1;    
    end
    
    if isfield(options,'Display') && isscalar(options.Display)
        Display = options.Display;
        disp(['User setting: Display = ',num2str(Display)])
        field_flag = 1;    
    end
    
    if isfield(options,'MaxIter') && isscalar(options.MaxIter)
        MaxIter = options.MaxIter;
        disp(['User setting: MaxIter = ',num2str(MaxIter)])
        field_flag = 1;
    end
    
    if isfield(options,'TolFun') && isscalar(options.TolFun)
        TolFun = options.TolFun;
        disp(['User setting: TolFun = ',num2str(TolFun)])
        field_flag = 1;
    end
    
    if isfield(options,'MaxAttempt') && isscalar(options.MaxAttempt)
        MaxAttempt = options.MaxAttempt;
        disp(['User setting: MaxAttempt = ',num2str(MaxAttempt)])
        field_flag = 1;
    end
    
    if isfield(options,'MaxDarts') && isscalar(options.MaxDarts)
        MaxDarts = options.MaxDarts;
        disp(['User setting: MaxDarts = ',num2str(MaxDarts)])
        field_flag = 1;
    end
    
    if isfield(options,'NelderMead') && isscalar(options.NelderMead)
        NelderMead = options.NelderMead;
        disp(['User setting: NelderMead = ',num2str(NelderMead)])
        field_flag = 1;
    end
    
    if isfield(options,'Minutes') && isscalar(options.Minutes)
        Minutes = options.Minutes;
        disp(['User setting: Minutes = ',num2str(Minutes)])
        field_flag = 1;
    end
    
    if isfield(options,'FinalCheck') && isscalar(options.FinalCheck)
        FinalCheck = options.FinalCheck;
        disp(['User setting: FinalCheck = ',num2str(FinalCheck)])
        field_flag = 1;
    end
    
    if isfield(options,'Rounding') && isscalar(options.Rounding)
        Rounding = options.Rounding;
        disp(['User setting: Rounding = ',num2str(Rounding)])
        field_flag = 1;
    end
    
    if field_flag == 0
        disp('Sorry, Unable to extract information from user supplied options. Defaults loaded.')
    end
end
   
   

% Convert string expression to function handle
if ischar(fun)    
    loc = strfind(fun,'=');
    if ~isempty(loc)
        fun = fun(loc+1:end);
    end
    
    fun = lower(fun);
    fun = strrep(fun,'exp','#1#');
    fun = strrep(fun,'log','#2#');
    fun = strrep(fun,'sin','#3#');
    fun = strrep(fun,'cos','#4#');
    fun = strrep(fun,'tan','#5#');
    fun = strrep(fun,'abs','#6#');
    fun = strrep(fun,'eye','#7#');
    fun = strrep(fun,'ones','#8#');
    fun = strrep(fun,'zeros','#9#');
    fun = strrep(fun,'sqrt','#10#');
    fun = strrep(fun,'max','#11#');
    fun = strrep(fun,'min','#12#');

    
    mask = isletter(fun);
    letter_set = unique(fun(mask));
    nletter = length(letter_set);
    if nletter == 0
        error('Sorry, variables in objective function cannot be identified.')
    end
    if nletter == 1
        for m = 1:20
            fun = strrep(fun,[letter_set,num2str(m)],['x(',num2str(m),')']);
        end
        fun = strrep(fun,letter_set,'x');
    else
        fun = strrep(fun,'x','$');
        letter_set = strrep(letter_set,'x','$');
        for m = 1:nletter
            fun = strrep(fun,letter_set(m),['x(',num2str(m),')']);
        end
    end
    
    fun = strrep(fun,'#1#','exp');
    fun = strrep(fun,'#2#','log');
    fun = strrep(fun,'#3#','sin');
    fun = strrep(fun,'#4#','cos');
    fun = strrep(fun,'#5#','tan');
    fun = strrep(fun,'#6#','abs');
    fun = strrep(fun,'#7#','eye');
    fun = strrep(fun,'#8#','ones');
    fun = strrep(fun,'#9#','zeros');
    fun = strrep(fun,'#10#','sqrt');
    fun = strrep(fun,'#11#','max');
    fun = strrep(fun,'#12#','min');
    
    fun = eval(['@(x)',fun,';']);
    disp('Objective function in string expression converted to an anonymous function:')
    disp(fun)
end
    

switch MethodGradient
    case 0
        disp('Gradient method: User supplied analytic gradients.')
        fun_grad = fun;
    case 1
        disp('Gradient method: Numerical gradients by forward difference.')
        fun_grad = @(x0) fun_grad1(x0,fun,DiffDelta);
    case 2
        disp('Gradient method: Numerical gradients by central difference.')
        fun_grad = @(x0) fun_grad2(x0,fun,DiffDelta);
    case 3
        disp('Gradient method: Numerical gradients by Richardson extrapolation I.')
        fun_grad = @(x0) fun_grad3(x0,fun,DiffDelta);
    case 4
        disp('Gradient method: Numerical gradients by Richardson extrapolation II.')
        fun_grad = @(x0) fun_grad4(x0,fun,DiffDelta);
    case 5
        disp('Gradient method: Numerical gradients by Richardson extrapolation III.')
        fun_grad = @(x0) fun_grad5(x0,fun,DiffDelta);
    case 6
        disp('Gradient method: Numerical gradients by Richardson extrapolation on central difference.')
        fun_grad = @(x0) fun_grad6(x0,fun,DiffDelta);
end

if exist('MAX_CORE','file') < 2
    error('Oooopse, a file called MAX_CORE is missing. Please download Climb High again.')
end

Max_flag = 1;
if nargout == 5
    [x,fval,exitflag,Gradient,Hessian] = MAX_CORE(fun_grad,x0,MethodGradient,Display,MaxIter,TolFun,MaxAttempt,MaxDarts,NelderMead,Minutes,FinalCheck,Rounding,Max_flag);
else
    [x,fval,exitflag,Gradient] = MAX_CORE(fun_grad,x0,MethodGradient,Display,MaxIter,TolFun,MaxAttempt,MaxDarts,NelderMead,Minutes,FinalCheck,Rounding,Max_flag);
end

end

%---------------------------
% Subfunction: fun_grad1
% Supplement forward gradients
%---------------------------
function [fval,gradient] = fun_grad1(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1);   
    return
end

if nargout > 1
    delta = delta * x0;
    delta(delta==0) = 1e-6;
    dim = length(x0);
    gradient = zeros(dim,1);
    y0 = fval;
    for m = 1:dim        
        x_temp = x0; x_temp(m) = x_temp(m) + delta(m);
        y1 = fun(x_temp);
        gradient(m) = (y1 - y0) / delta(m);        
    end
    gradient(~isfinite(gradient)) = 0;
end
end

%---------------------------
% Subfunction: fun_grad2
% Supplement central gradients
%---------------------------
function [fval,gradient] = fun_grad2(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1); 
    return
end

if nargout > 1
    delta = delta * x0;
    delta(delta==0) = 1e-6;
    dim = length(x0);
    gradient = zeros(dim,1);    
    for m = 1:dim
        delta_use = delta(m);
        x_temp = x0; 
        x_temp(m) = x_temp(m) + delta_use;
        y_up = fun(x_temp);
        x_temp(m) = x_temp(m) - 2 * delta_use;
        y_down = fun(x_temp);
        gradient(m) = (y_up - y_down) / (2*delta_use);        
    end    
    gradient(~isfinite(gradient)) = 0;
end
end

%---------------------------
% Subfunction: fun_grad3
% Supplement Richardson I gradients
%---------------------------
function [fval,gradient] = fun_grad3(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1);
    return
end

if nargout > 1
    delta = sqrt(delta) * x0;
    delta(delta==0) = 1e-3;
    dim = length(x0);
    gradient = zeros(dim,1);
    y0 = fval;
    for m = 1:dim
        delta_use = delta(m);
        x_temp = x0; 
        x_temp(m) = x_temp(m) + delta_use;
        y2 = fun(x_temp);
        x_temp(m) = x_temp(m) - 0.5 * delta_use;
        y1 = fun(x_temp);       
        
        d10 = (y1 - y0) / (0.5*delta_use);
        d20 = (y2 - y0) / delta_use;
        gradient(m) = 2 * d10 - d20;        
    end
    gradient(~isfinite(gradient)) = 0;
end
end

%---------------------------
% Subfunction: fun_grad4
% Supplement Richardson II gradients
%---------------------------
function [fval,gradient] = fun_grad4(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1);
    return
end

if nargout > 1
    delta = sqrt(delta) * x0;
    delta(delta==0) = 1e-3;
    dim = length(x0);
    gradient = zeros(dim,1);
    y0 = fval;
    for m = 1:dim
        delta_use = delta(m);
        x_temp = x0; 
        x_temp(m) = x_temp(m) + 0.25*delta_use;       
        y1 = fun(x_temp);
        x_temp(m) = x_temp(m) + 0.25*delta_use;
        y2 = fun(x_temp);
        x_temp(m) = x_temp(m) + 0.5*delta_use;
        y3 = fun(x_temp);        
        d10 = (y1 - y0) ./ (0.25*delta_use);
        d20 = (y2 - y0) ./ (0.5*delta_use);
        d30 = (y3 - y0) ./ delta_use;
        gradient(m) = d30/3 - 2*d20 + 8/3*d10;        
    end
    gradient(~isfinite(gradient)) = 0;
end
end

%---------------------------
% Subfunction: fun_grad5
% Supplement Richardson III gradients
%---------------------------
function [fval,gradient] = fun_grad5(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1); 
    return
end

if nargout > 1
    delta = sqrt(delta) * x0;
    delta(delta==0) = 1e-3;
    dim = length(x0);
    gradient = zeros(dim,1);
    y0 = fval;
    for m = 1:dim
        delta_use = delta(m);
        x_temp = x0; 
        x_temp(m) = x_temp(m) + 0.125*delta_use; 
        y1 = fun(x_temp);
        x_temp(m) = x_temp(m) + 0.125*delta_use; 
        y2 = fun(x_temp);
        x_temp(m) = x_temp(m) + 0.25*delta_use;
        y3 = fun(x_temp);
        x_temp(m) = x_temp(m) + 0.5*delta_use;
        y4 = fun(x_temp);        
        d10 = (y1 - y0) ./ (0.125*delta_use);
        d20 = (y2 - y0) ./ (0.25*delta_use);
        d30 = (y3 - y0) ./ (0.5*delta_use);
        d40 = (y4 - y0) ./ delta_use;        
        L1_one = 2 * d30 - d40;
        L1_half = 2 * d20 - d30;
        L1_quarter = 2 * d10 - d20;        
        L2_one = (4 * L1_half - L1_one)/3;
        L2_half = (4 * L1_quarter - L1_half)/3;        
        L3_one = (8 * L2_half - L2_one)/7;
        gradient(m) = L3_one;
    end
    gradient(~isfinite(gradient)) = 0;
end
end

%---------------------------
% Subfunction: fun_grad6
% Supplement Richardson on central gradients
%---------------------------
function [fval,gradient] = fun_grad6(x0,fun,delta)
fval = fun(x0);
if ~isfinite(fval) || ~isreal(fval)
    fval = -Inf;
    gradient = zeros(length(x0),1);
    return
end

if nargout > 1
    delta = sqrt(delta) * x0;
    delta(delta==0) = 1e-3;
    dim = length(x0);
    gradient = zeros(dim,1);    
    for m = 1:dim
        delta_use = delta(m);
        x_temp = x0; 
        x_temp(m) = x_temp(m) + delta_use;
        y_up = fun(x_temp);
        x_temp(m) = x_temp(m) - 2 * delta_use;
        y_down = fun(x_temp);
        d_long = (y_up - y_down) / (2*delta_use); 
        
        x_temp = x0; 
        x_temp(m) = x_temp(m) + 0.5 * delta_use;
        y_up = fun(x_temp);
        x_temp(m) = x_temp(m) - delta_use;
        y_down = fun(x_temp);
        d_short = (y_up - y_down) / delta_use;         
        gradient(m) = (4*d_short - d_long)/3;        
    end    
    gradient(~isfinite(gradient)) = 0;
end
end