clear all
clc

% This routine calculates optimal portfolio in Qian(2011) 
% "Bayesian Portfolio Selection in Markov Switching Gaussian Mixture Model"

myseed = 20111218;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
end

% Load the data
[price,header] = xlsread('StockPrice.xls');
lnP = log(price);
R = diff(lnP)*100;
[nobs,nvar] = size(R);

% Estimate the Markov Switching Gaussian Mixture Model
nregime = 3;
nstate = 3;
ndraws = 50000;
burn_in = ndraws * 0.5;
permute_flag = 1;
[Mu_draws,Sigma_draws,Markov_draws,Initial_prob_draws,MixProb_draws,Regime_draws,Rf] = HMM_GM_EQ(R,nregime,nstate,ndraws,burn_in,permute_flag);

% Calculate mean-variance frontier
Rf_mean = mean(Rf,2);
Rf_cov = cov(Rf');
Rp =  -0.08:0.001:0.15;
[Wp_BAY,Vp_BAY] = MV_FRONT(Rf_mean,Rf_cov,Rp);
[Wp_CE,Vp_CE] = MV_FRONT(mean(R),cov(R),Rp);
plot(Vp_BAY,Rp,'-b',Vp_CE,Rp,':r')
legend('Bayesian','Certainty Equiv.','Location','East')
xlabel('Variance')
ylabel('Expected returns')

% Compute moments of the predictive returns
% so as to approximate the expected utility
[nvar,ndraws] = size(Rf);
M1 = mean(Rf,2);
Rf_demean = Rf - M1(:,ones(ndraws,1));
M2 = (Rf_demean * Rf_demean') ./ ndraws;
M3 = zeros(nvar,nvar^2);
M4 = zeros(nvar,nvar^3);
for t = 1:ndraws
    Rf_use = Rf_demean(:,t);
    power3 = kron(Rf_use * Rf_use', Rf_use');
    power4 = kron(power3, Rf_use');
    M3 = M3 + power3;
    M4 = M4 + power4;
end
M3 = M3 ./ ndraws;
M4 = M4 ./ ndraws; 

% Optimal portfolio by maximizing expected utility (moments approximation)
% Two cases: short sale allowed, short sale not allowed
w0 = 1/(nvar)*ones(nvar-1,1);
ARA = 1:2:11;
ntype = length(ARA);
options = optimset('LargeScale','off','Display','off');
options_high = struct('Display',0);
W_optimal = zeros(nvar-1,ntype);
W_optimal_pos = zeros(nvar-1,ntype);
for m = 1:ntype
    try
        W_optimal(:,m) = fminunc(@(w)-EU_MAX(w,ARA(m),M1,M2,M3,M4,1),w0,options);
    catch
        W_optimal(:,m) = MAXIMIZE(@(w)EU_MAX(w,ARA(m),M1,M2,M3,M4,1),w0,options_high);
    end
    while abs(any(W_optimal(:,m))) > 1
        disp('Reoptimize')
        W_optimal(:,m) = MAXIMIZE(@(w)EU_MAX(w,ARA(m),M1,M2,M3,M4,1),rand(nvar-1,1)/nvar,options_high);
    end
    
    try
        W_optimal_pos(:,m) = fminunc(@(w)-EU_MAX(w,ARA(m),M1,M2,M3,M4,0),w0,options);
    catch
        W_optimal_pos(:,m) = MAXIMIZE(@(w)EU_MAX(w,ARA(m),M1,M2,M3,M4,0),w0,options_high);
    end
    while abs(any(W_optimal(:,m))) > 1
        disp('Reoptimize')
        W_optimal_pos(:,m) = MAXIMIZE(@(w)EU_MAX(w,ARA(m),M1,M2,M3,M4,0),rand(nvar-1,1)/nvar,options_high);
    end       
    
end
W_optimal = [W_optimal;1-sum(W_optimal)]*100;
W_optimal_pos = [W_optimal_pos;1-sum(W_optimal_pos)]*100;

disp('EU-max portfolio ')
for m = 1:nvar    
    fprintf('R(%1.0f) \t',m)
    fprintf('%8.2f', W_optimal(m,:))
    fprintf('\n')
end
fprintf('\n')

disp('EU-max portfolio without shortsell ')
for m = 1:nvar    
    fprintf('R(%1.0f) \t',m)
    fprintf('%8.2f', W_optimal_pos(m,:))
    fprintf('\n')
end
fprintf('\n')

% Gererate a report
header = header(1,2:end);
Rf_var = diag(Rf_cov);
Rf_plus = mean(Rf>0,2);
xlswrite('Portfolio selection',{'Pred. Mean','Pred. Var','Pr(R>0)'},'Sheet1','B1')
xlswrite('Portfolio selection',header','Sheet1','A2')
xlswrite('Portfolio selection',[Rf_mean,Rf_var,Rf_plus],'Sheet1','B2')

risk_averse = cell(1,ntype);
for m = 1:ntype
    risk_averse{m} = ['a  =  ',num2str(m)];
end
xlswrite('Portfolio selection',risk_averse,'Sheet2','B1')
xlswrite('Portfolio selection',header','Sheet2','A2')
xlswrite('Portfolio selection',W_optimal,'Sheet2','B2')

xlswrite('Portfolio selection',risk_averse,'Sheet3','B1')
xlswrite('Portfolio selection',header','Sheet3','A2')
xlswrite('Portfolio selection',W_optimal_pos,'Sheet3','B2')


   
